@@ -1,9 +1,9 @@
-# -*- mode: cperl; coding: latin-2 -*-
+# -*- mode: cperl -*-
 
 #
 # Author: Slaven Rezic
 #
-# Copyright (C) 2006,2007,2008,2009 Slaven Rezic. All rights reserved.
+# Copyright (C) 2006,2007,2008,2009,2010,2015 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -14,20 +14,13 @@
 package Kwalify;
 
 use strict;
+use warnings;
 
 use base qw(Exporter);
 use vars qw(@EXPORT_OK $VERSION);
 @EXPORT_OK = qw(validate);
 
-$VERSION = '1.20_51';
-
-BEGIN {
-    if ($] < 5.006) {
-	$INC{"warnings.pm"} = 1;
-	*warnings::import = sub { };
-	*warnings::unimport = sub { };
-    }
-}
+$VERSION = '1.22_90';
 
 sub validate ($$) {
     my($schema, $data) = @_;
@@ -77,6 +70,7 @@ sub _validate {
 
 sub _additional_rules {
     my($self, $schema, $data, $path) = @_;
+    no warnings 'uninitialized'; # legal undef values may happen everywhere
     for my $schema_key (keys %$schema) {
 	if (defined $schema->{$schema_key}) {
 	    if ($schema_key eq 'pattern') {
@@ -118,7 +112,6 @@ sub _additional_rules {
 		if (!UNIVERSAL::isa($schema->{enum}, 'ARRAY')) {
 		    $self->_die("'enum' must be an array");
 		}
-		no warnings 'uninitialized'; # undef amongst the enum values may be intended!
 		my %valid = map { ($_,1) } @{ $schema->{enum} };
 		if (!exists $valid{$data}) {
 		    $self->_error("'$data': invalid " . _base_path($path) . " value");
@@ -342,12 +335,13 @@ sub validate_map {
 	    $self->_die("Expected subschema (a hash)");
 	}
 	my $required = _get_boolean($subschema->{required});
-	if (!exists $data->{$key}) {
+	if (!defined $data->{$key}) {
 	    if ($required) {
 		$self->{path} = $path;
 		$self->_error("Expected required key '$key'");
 		next;
 	    } else {
+		$seen_key{$key}++;
 		next;
 	    }
 	}
@@ -415,8 +409,6 @@ sub _get_boolean {
 1;
 __END__
 
-=encoding iso-8859-2
-
 =head1 NAME
 
 Kwalify - Kwalify schema for data structures
@@ -432,7 +424,7 @@ Typically used together with YAML or JSON:
   validate(YAML::LoadFile($schema_file), YAML::LoadFile($data_file));
 
   use JSON;
-  validate(jsonToObj($schema_data), jsonToObj($data));
+  validate(decode_json($schema_data), decode_json($data));
 
 =head1 DESCRIPTION
 
@@ -544,20 +536,19 @@ show a path-like expression to the data causing the error.
 
 =head1 AUTHOR
 
-Slaven Rezi, E<lt>srezic@cpan.orgE<gt>
+Slaven ReziE<x0107>, E<lt>srezic@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2006,2007,2008,2009 by Slaven Rezi
+Copyright (C) 2006,2007,2008,2009,2010,2015 by Slaven ReziE<x0107>
 
 This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.8.8 or,
-at your option, any later version of Perl 5 you may have available.
+it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
 L<pkwalify>, L<kwalify(1)>.
 
-Other non-XML schema languages: L<http://rjbs.manxome.org/rx/>
+Other non-XML schema languages: L<http://rx.codesimply.com/>
 
 =cut
